package com.ruoyi.ss.store.service.impl;

import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.SmDeviceQuery;
import com.ruoyi.ss.device.domain.SmDeviceVo;
import com.ruoyi.ss.device.service.ISmDeviceService;
import com.ruoyi.ss.store.domain.Store;
import com.ruoyi.ss.store.domain.StoreQuery;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.mapper.StoreMapper;
import com.ruoyi.ss.store.service.IStoreService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 设备分组Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
public class StoreServiceImpl implements IStoreService
{
    @Autowired
    private StoreMapper storeMapper;

    @Autowired
    private ISmDeviceService smDeviceService;

    /**
     * 查询设备分组
     *
     * @param storeId 设备分组主键
     * @return 设备分组
     */
    @Override
    public StoreVo selectSmStoreById(Long storeId)
    {
        return storeMapper.selectSmStoreById(storeId);
    }

    /**
     * 查询设备分组列表
     *
     * @param store 设备分组
     * @return 设备分组
     */
    @Override
    public List<StoreVo> selectSmStore(StoreQuery store)
    {
        return storeMapper.selectSmStoreList(store);
    }

    /**
     * 新增设备分组
     *
     * @param store 设备分组
     * @return 结果
     */
    @Override
    public int insertSmStore(Store store)
    {
        store.setCreateTime(DateUtils.getNowDate());
        return storeMapper.insertSmStore(store);
    }

    /**
     * 修改设备分组
     *
     * @param store 设备分组
     * @return 结果
     */
    @Override
    public int updateSmStore(Store store)
    {
        this.validate(Collections.singletonList(store), true);
        return storeMapper.updateSmStore(store);
    }

    /**
     * 批量删除设备分组
     *
     * @param storeIds 需要删除的设备分组主键
     * @return 结果
     */
    @Override
    public int deleteSmStoreByIds(Long[] storeIds)
    {
        this.validatePreDelete(Arrays.asList(storeIds));
        return storeMapper.deleteSmStoreByIds(storeIds);
    }

    /**
     * 删除设备分组信息
     *
     * @param storeId 设备分组主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteSmStoreById(Long storeId) {
        // 删除校验
        this.validatePreDelete(storeId);

        // 删除分组
        return storeMapper.deleteSmStoreById(storeId);
    }

    /**
     * 校验数据
     */
    private void validate(List<Store> list, boolean update) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (LoginType.FRONT.equals(loginUser.getLoginType())) {
            if (update) {
                List<StoreVo> stores = storeMapper.selectSmStoreByIds(list.stream().map(Store::getStoreId).distinct().collect(Collectors.toList()));
                Long userId = SecurityUtils.getUserId();
                stores.forEach(item -> {
                    ServiceUtil.assertion(!Objects.equals(item.getUserId(), userId), String.format("存在不属于您的分组：%s", item.getName()));
                });
            }
        }
    }

    @Override
    @Transactional
    public boolean changeSort(List<Store> list) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(list), "修改排序的列表不允许为空");
        this.validate(list, true);

        List<Store> updateList = new ArrayList<>();
        for (Store item : list) {
            Store store = new Store();
            store.setStoreId(item.getStoreId());
            store.setGroupSort(item.getGroupSort());
            updateList.add(store);
        }
        storeMapper.batchUpdate(updateList);
        return true;
    }

    @Override
    public List<StoreVo> listCount(Store store) {
        List<StoreVo> list = new ArrayList<>();

        // 查询全部设备数据
        SmDeviceQuery deviceDto = new SmDeviceQuery();
        deviceDto.setUserId(store.getUserId());
        int count = smDeviceService.selectCount(deviceDto);
        StoreVo all = new StoreVo();
        all.setDeviceCount(count);
        all.setName("全部");
        list.add(all);

        // 查询各各分组的设备数据
        list.addAll(storeMapper.selectSmStoreList(store));
        return list;
    }

    @Override
    public StoreVo selectDefaultStore(Long userId) {
        if (userId == null) {
            return null;
        }
        StoreQuery dto = new StoreQuery();
        dto.setIsDefault(true);
        dto.setUserId(userId);
        List<StoreVo> list = selectSmStore(dto);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional
    public StoreVo insertDefaultStore(Long userId) {
        StoreVo store = new StoreVo();
        store.setUserId(userId);
        store.setName("未分组");
        store.setGroupSort(1);
        store.setIsDefault(true);
        if (this.insertSmStore(store) == 1)  {
            return store;
        }
        return null;
    }

    /**
     * ids逻辑删除
     *
     * @param ids id列表
     */
    @Override
    public int logicDel(List<Long> ids) {
        return storeMapper.logicDel(ids);
    }

    private void validatePreDelete(List<Long> storeIds) {
        if (!CollectionUtils.isEmpty(storeIds)) {
            for (Long storeId : storeIds) {
                this.validatePreDelete(storeId);
            }
        }
    }

    // 删除校验
    private void validatePreDelete(Long storeId) {
        Store dto = new Store();
        if (LoginType.FRONT.equals(SecurityUtils.getLoginType())) {
            // 前台用户，判断是不是他的分组
            dto.setUserId(SecurityUtils.getUserId());
        }
        StoreVo store = storeMapper.selectSmStoreById(storeId);
        ServiceUtil.assertion(store == null, "分组不存在，请刷新后重试");
        ServiceUtil.assertion(store.getIsDefault(), "默认分组不允许删除");

        // 查询分组下是否含有设备
        List<SmDeviceVo> deviceList = smDeviceService.selectSmDeviceByStoreId(storeId);
        ServiceUtil.assertion(!CollectionUtils.isEmpty(deviceList), "该分组下存在设备，不允许删除");
    }


}
